#!/bin/bash

AWK="/usr/bin/awk"
ECHO="/bin/echo"
GREP="/usr/bin/grep"
ID="/usr/xpg4/bin/id"
LS="/bin/ls"
SED="/usr/bin/sed"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
UNAME="/bin/uname"
WC="/bin/wc"

OS="unknown"
OS_VERSION="unknown"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_basic_services {
   print_banner "Checking Services"
   print_divider
   case ${OS_VERSION} in
      "10")
         # Check for basic/infrequently used/insecure services
         SERVICES="chargen:dgram chargen:stream daytime:dgram daytime:stream"
         SERVICES="${SERVICES} discard:dgram discard:stream echo:dgram echo:stream"
         SERVICES="${SERVICES} time:dgram time:stream rpc/rex:default rexec:default"
         SERVICES="${SERVICES} uucp:default comsat:default rpc/spray:default"
         SERVICES="${SERVICES} rpc/wall:default tname:default talk:default finger:default"
         SERVICES="${SERVICES} rpc/rstat:default rpc/rusers:default rpc/ocfserv:default"
         SERVICES="${SERVICES} login:eklogin login:klogin shell:kshell login:rlogin"
         SERVICES="${SERVICES} shell:default" 
      
         SVC_COUNT=0
      
         for SERVICE in ${SERVICES}; do
           if [ "$( ${SVCS} -a | ${GREP} "${SERVICE}" | ${WC} -l )" -eq "0" ]; then
             print_ok "${SERVICE} not present on this system"
             return
           else
            STATE=$( ${SVCS} svc:/network/${SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
            if [ "${STATE}" != "disabled" ]; then
               print_warn "Service \"svc:/network/${SERVICE}\" is [${STATE}]"
               (( SVC_COUNT = SVC_COUNT + 1 ))
            fi
           fi
         done
      
         if [ "${SVC_COUNT}" -gt "0" ]; then
            print_warn "${SVC_COUNT} questionable services found"
         else
            print_ok "No questionable services found"
         fi
         ;;
   esac
}

function check_rpc_services {
   print_banner "Checking RPC Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          # svc:/network/rpc/bind:default
          if [ "$( ${SVCS} -a | ${GREP} "svc:/network/rpc/bind:default" | ${WC} -l )" -eq "0" ]; then
             print_ok "RPC Bind not present"
             return
          fi
          RPC_BIND_STATUS=$( ${SVCS} svc:/network/rpc/bind:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${RPC_BIND_STATUS}" != "disabled" ]; then
             print_warn "Service \"svc:/network/rpc/bind:default\" is [${RPC_BIND_STATUS}]"
             RPC_DEPENDENCIES=$( ${SVCS} -H -D svc:/network/rpc/bind:default | ${AWK} '{print $1, $NF }' | ${GREP} -v "^disabled" )
             print_info "The following online services are dependent on this service:"
             ${ECHO} "${RPC_DEPENDENCIES}"
             print_info "Checking svc:/network/rcp/bind:default for tcpwrappers support"
             RPC_BIND_TCPD=$( ${SVCCFG} -s rpc/bind listprop config/enable_tcpwrappers | ${AWK} '{print $NF}' )
             if [ "${RPC_BIND_TCPD}" = "false" ]; then
                print_warn "svc:/network/rpc/bind:default tcpwrappers support is not enabled"
                print_info "Consider enabling tcpwrappers support with the following commands:"
                print_info "${SVCCFG} -s rpc/bind setprop config/enable_tcpwrappers = true"
                print_info "${SVCADM} refresh rpc/bind"
             else
                print_ok "svc:/network/rpc/bind:default has tcpwrappers support enabled"
             fi
          else
             print_ok "Service \"svc:/network/rpc/bind:default\" is disabled"
          fi
          # svc:/network/rpc/keyserv:default          
          SECURE_RPC_STATUS=$( ${SVCS} svc:/network/rpc/keyserv:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${SECURE_RPC_STATUS}" != "disabled" ]; then
             print_warn "Service \"svc:/network/rpc/bind:default\" is [${SECURE_RPC_STATUS}]"
          else
             print_ok "Service \"svc:/network/rpc/keyserv:default\" is disabled"
          fi
          ;;
   esac
}

function check_nis_services {
   print_banner "Checking NIS/NIS+ Services"
   print_divider
   ALL_DISABLED=0
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "nis" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No NIS/NIS+ Services Installed"
             return
          fi
          for NIS_SERVICE in $( ${SVCS} -a | ${GREP} "nis" | ${AWK} '{print $NF}' ); do
              NIS_SERVICE_STATUS=$( ${SVCS} ${NIS_SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
              if [ "${NIS_SERVICE_STATUS}" != "disabled" ]; then
                 print_warn "Service \"${NIS_SERVICE}\" is [${NIS_SERVICE_STATUS}]"
                 ALL_DISABLED=1
              fi
          done
          if [ "${ALL_DISABLED}" -eq "0" ]; then
             print_ok "All NIS/NIS+ related services disabled"
          fi 
          ;;
   esac
}

function check_ldap_services {
   print_banner "Checking LDAP Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "ldap" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No LDAP Services Installed"
             return
          fi
          LDAP_STATUS=$( ${SVCS} svc:/network/ldap/client:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${LDAP_STATUS}" != "disabled" ]; then
             print_warn "Service \"svc:/network/ldap/client:default\" is [${LDAP_STATUS}]"
          else
             print_ok "LDAP cache manager is disabled" 
          fi
          ;;
   esac
}

function check_kerberos_services {
   print_banner "Checking Kerberos/GSS Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "security" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No Kerberos/GSS Services Installed"
             return
          fi
          SERVICES="security/kadmin:default security/krb5kdc:default security/krb5_prop:default security/ktkt_warn:default"
          SERVICES="${SERVICES} rpc/gss:default"
          ALL_DISABLED=0
          for SERVICE in ${SERVICES}; do
             if [ "$( ${SVCS} -a | ${GREP} "${SERVICE}" | ${WC} -l )" -eq "0" ]; then
                print_ok "${SERVICE} not present"
             else
                STATE=$( ${SVCS} svc:/network/${SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
                if [ "${STATE}" != "disabled" ]; then
                   print_warn "Service \"svc:/network/${SERVICE}\" is [${STATE}]"
                   ALL_DISABLED=1
                fi 
             fi
          done
          if [ ${ALL_DISABLED} -eq "0" ]; then
             print_ok "All Kerberos services are disabled"
          fi
          ;;
   esac
}

function check_dtlogin_services {
   print_banner "Checking if dtlogin is running"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "svc:/network/rpc-100083_1/rpc_tcp:default" | ${WC} -l )" -eq "0" ]; then
             print_ok "svc:/network/rpc-100083_1/rpc_tcp:default is not present"
          else
             RPC_100083_STATE=$( ${SVCS} svc:/network/rpc-100083_1/rpc_tcp:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
             if [ "${RPC_100083_STATE}" != "disabled" ]; then
                print_warn "Service \"svc:/network/rpc-100083_1/rpc_tcp:default\" is [${RPC_100083_STATE}]"
             else
                print_ok "Service \"svc:/network/rpc-100083_1/rpc_tcp:default\" is disabled"
             fi
          fi 
          ${LS} /etc/rc2.d/S99dtlogin >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "dtlogin is set to start in runlevel 2"
             print_info "mv /etc/rc2.d/S99dtlogin /etc/rc2.d/.NOS99dtlogin and reboot"
          else
             print_ok "dtlogin is not set to start in runlevel 2"
          fi
          ;;
   esac
}

function check_smc_services {
   print_banner "Checking Solaris Management Console services"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${LS} /etc/rc2.d/S90wbem >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "wbem is set to start in runlevel 2"
             print_info "mv /etc/rc2.d/S90wbem /etc/rc2.d/.NOS90wbem and reboot"
          else
             print_ok "wbems not set to start in runlevel 2"
          fi
          ${LS} /etc/rc2.d/S90webconsole >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "webconsole is set to start in runlevel 2"
             print_info "mv /etc/rc2.d/S90webconsole /etc/rc2.d/.NOS90webconsole and reboot"
          else
             print_ok "dtlogin is not set to start in runlevel 2"
          fi
          ;;
   esac
}

function check_volmgt_services {
   print_banner "Checking volmgt Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "smserver" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No volmgt Services Installed"
             return
          fi
          VOLMGT_STATE=$( ${SVCS} svc:/network/rpc/smserver:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${VOLMGT_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/rpc/smserver:default\" is [${VOLMGT_STATE}]"
          else
             print_ok "Service \"svc:/network/rpc/smserver:default\" is disabled"
          fi
          ;;
   esac
}

function check_samba_services {
   print_banner "Checking SAMBA Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${LS} /etc/rc3.d/S90samba >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "samba is set to start in runlevel 3"
             print_info "mv /etc/rc3.d/S90samba /etc/rc3.d/.NOS90samba and reboot"
          else
             print_ok "samba not set to start in runlevel 3"
          fi
          ;;
   esac
}

function check_nfs_services {
   print_banner "Checking NFS Services"
   print_divider
   ALL_DISABLED=0
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "nfs" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No NFS Services Installed"
             return
          fi
          for NFS_SERVICE in $( ${SVCS} -a | ${GREP} "nfs" | ${AWK} '{print $NF}' ); do
             SERVICE_STATE=$( ${SVCS} ${NFS_SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
             if [ "${SERVICE_STATE}" != "disabled" ]; then
                print_warn "Service \"${NFS_SERVICE}\" is [${SERVICE_STATE}]"
                ALL_DISABLED=1
             fi
          done
          if [ "${ALL_DISABLED}" -eq "0" ]; then 
             print_ok "All NFS related services disabled"
          fi
          ;;
   esac
}

function check_automount_services {
   print_banner "Checking Automount Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "autofs" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No Automount Services Installed"
             return
          fi
          AUTOMOUNT_STATE=$( ${SVCS} svc:/system/filesystem/autofs:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${AUTOMOUNT_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/system/filesystem/autofs:default\" is [${AUTOMOUNT_STATE}]"
          else
             print_ok "Service \"svc:/system/filesystem/autofs:default\" is disabled"
          fi
          ;;
    esac
}

function check_telnet_services {
   print_banner "Checking Telnet Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "telnet" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No Telnet Services Installed"
             return
          fi
          TELNET_STATE=$( ${SVCS} svc:/network/telnet:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${TELNET_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/telnet:default\" is [${TELNET_STATE}]"
          else
             print_ok "Service \"svc:/network/telnet:default\" is disabled"
          fi
          ;;
   esac
}

function check_ftp_services {
   print_banner "Checking FTP Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "ftp" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No FTP Services Installed"
             return
          fi
          FTP_STATE=$( ${SVCS} svc:/network/ftp:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${FTP_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/ftp:default\" is [${FTP_STATE}]"
          else
             print_ok "Service \"svc:/network/ftp:default\" is disabled"
          fi
          ;;
   esac
}

function check_boot_services {
   print_banner "Checking Boot Services"
   print_divider
   ALL_DISABLED=0
   case ${OS_VERSION} in 
      "10")
          BOOT_SERVICES="rarp:default rpc/bootparams:default"
          for BOOT_SERVICE in ${BOOT_SERVICES}; do
             if [ "$( ${SVCS} -a | ${GREP} "${BOOT_SERVICE}" | ${WC} -l )" -eq "0" ]; then
                print_ok "${BOOT_SERVICE} not present"
             else
                STATE=$( ${SVCS} svc:/network/${BOOT_SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
                if [ "${STATE}" != "disabled" ]; then
                   print_warn "Service \"svc:/network/${SERVICE}\" is [${STATE}]" 
                   ALL_DISABLED=1   
                fi
             fi
          done
          if [ "${ALL_DISABLED}" -eq "0" ]; then 
             print_ok "All Boot services are disabled"
          fi
          ;;
   esac
}

function check_dhcp_services {
   print_banner "Checking DHCP Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "dhcp" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No DHCP Services Installed"
             return
          fi
          DHCP_STATE=$( ${SVCS} svc:/network/dhcp-server:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${DHCP_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/dhcp-server:default\" is [${DHCP_STATE}]"
          else
             print_ok "Service \"svc:/network/dhcp-server:default\" is disabled"
          fi
          ;;
   esac
}

function check_dns_services {
   print_banner "Checking DNS Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "dns" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No DNS Services Installed"
             return
          fi
          DNS_STATE=$( ${SVCS} svc:/network/dns/server:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${DNS_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/dns/server:default\" is [${DNS_STATE}]"
          else
             print_ok "Service \"svc:/network/dns/server:default\" is disabled"
          fi
          ;;
   esac
}

function check_tftp_services {
   print_banner "Checking TFTP Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "tftp" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No TFTP Services Installed"
             return
          fi
          TFTP_STATE=$( ${SVCS} svc:/network/tftp:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${TFTP_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/tftp:default\" is [${TFTP_STATE}]"
          else
             print_ok "Service \"svc:/network/tftp:default\" is disabled"
          fi
          ;;
   esac
}

function check_print_services {
   print_banner "Checking Print Services"
   print_divider
   ALL_DISABLED=0
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "print" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No Print Services Installed"
             return
          fi
          PRINT_SERVICES=$( ${SVCS} -a | ${GREP} "print" | ${AWK} '{print $NF}' )
          for PRINT_SERVICE in ${PRINT_SERVICES}; do
             PRINT_STATE=$( ${SVCS} ${PRINT_SERVICE} | ${SED} -n '$p' | ${AWK} '{print $1}' )
             if [ "${PRINT_STATE}" != "disabled" ]; then 
                print_warn "Service \"${PRINT_SERVICE}\" is [${PRINT_STATE}]"
                ALL_DISABLED=1
             fi
          done
          if [ "${ALL_DISABLED}" -eq "0" ]; then 
             print_ok "All Print Services are disabled"
          fi
          ;;
   esac
}

function check_web_services {
   print_banner "Checking Web Services"
   print_divider
   case ${OS_VERSION} in
      "10")
          if [ "$( ${SVCS} -a | ${GREP} "apache2" | ${WC} -l )" -eq "0" ]; then 
             print_ok "No Apache2 Services Installed"
          else
             WEB_STATE=$( ${SVCS} svc:/network/http:apache2 | ${SED} -n '$p' | ${AWK} '{print $1}' )
             if [ "${WEB_STATE}" != "disabled" ]; then
                print_warn "Service \"svc:/network/http:apache2\" is [${WEB_STATE}]"
             else
                print_ok "Service \"svc:/network/http:apache2\" is disabled"
             fi
          fi
          ${LS} /etc/rc3.d/S50apache >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "apache is set to start in runlevel 3"
             print_info "mv /etc/rc3.d/S50apache /etc/rc3.d/.NOS50apache and reboot"
          else
             print_ok "apache not set to start in runlevel 3"
          fi
          ${LS} /etc/rc2.d/S42ncakmod >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "ncakmod is set to start in runlevel 2"
             print_info "mv /etc/rc2.d/S42ncakmod /etc/rc2.d/.NOS42ncakmod and reboot"
          else
             print_ok "ncakmod not set to start in runlevel 2"
          fi
          ${LS} /etc/rc2.d/S94ncalogd >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "ncalogd is set to start in runlevel 2"
             print_info "mv /etc/rc2.d/S94ncalogd /etc/rc2.d/.NOS94ncalogd and reboot"
          else
             print_ok "ncalogd not set to start in runlevel 2"
          fi
          ;;
   esac
}

function check_snmp_services {
   print_banner "Checking SNMP Service"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${LS} /etc/rc3.d/S82initsma >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then
             print_warn "initsma is set to start in runlevel 3"
             print_info "mv /etc/rc3.d/S82initsma /etc/rc3.d/.NOS82initsma and reboot"
          else
             print_ok "initsma not set to start in runlevel 3"
          fi
          ;;
   esac
}

function check_inetd {
   print_banner "Checking INETD"
   print_divider
   case ${OS_VERSION} in
      "10")
          INETD_STATE=$( ${SVCS} svc:/network/inetd:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
          if [ "${INETD_STATE}" != "disabled" ]; then
             print_warn "Service \"svc:/network/inetd:default\" is [${INETD_STATE}]"
          else
             print_ok "Service \"svc:/network/inetd:default\" is disabled"
          fi
          ;;
   esac
}

check_os
check_user
print_divider
check_basic_services
print_divider
check_rpc_services
print_divider
check_nis_services
print_divider
check_ldap_services
print_divider
check_kerberos_services
print_divider
check_dtlogin_services
print_divider
check_smc_services
print_divider
check_volmgt_services
print_divider
check_samba_services
print_divider
check_nfs_services
print_divider
check_automount_services
print_divider
check_telnet_services
print_divider
check_ftp_services
print_divider
check_boot_services
print_divider
check_dhcp_services
print_divider
check_dns_services
print_divider
check_tftp_services
print_divider
check_print_services
print_divider
check_web_services
print_divider
check_snmp_services
print_divider
# We could check SVM services - but we use them, so we won't bother!
check_inetd

print_divider

exit 0
